@@ -651,7 +651,7 @@ static void event_process(const XEvent *ec, gpointer data)
     else if (e->type == MappingNotify) {
         /* keyboard layout changes for modifier mapping changes. reload the
            modifier map, and rebind all the key bindings as appropriate */
-        ob_debug("Kepboard map changed. Reloading keyboard bindings.\n");
+        ob_debug("Keyboard map changed. Reloading keyboard bindings.\n");
         ob_set_state(OB_STATE_RECONFIGURING);
         modkeys_shutdown(TRUE);
         modkeys_startup(TRUE);
@@ -1276,7 +1276,7 @@ static void event_handle_client(ObClient *client, XEvent *e)
                                        it can happen now when the window is on
                                        another desktop, but we still don't
                                        want it! */
-        client_activate(client, FALSE, TRUE, TRUE, TRUE);
+        client_activate(client, FALSE, FALSE, TRUE, TRUE, TRUE);
         break;
     case ClientMessage:
         /* validate cuz we query stuff off the client here */
@@ -1323,10 +1323,7 @@ static void event_handle_client(ObClient *client, XEvent *e)
                        (e->xclient.data.l[0] == 2 ? "user" : "INVALID"))));
             /* XXX make use of data.l[2] !? */
             if (e->xclient.data.l[0] == 1 || e->xclient.data.l[0] == 2) {
-                /* don't use the user's timestamp for client_focus, cuz if it's
-                   an old broken timestamp (happens all the time) then focus
-                   won't move even though we're trying to move it
-                  event_curtime = e->xclient.data.l[1];*/
+                event_curtime = e->xclient.data.l[1];
                 if (e->xclient.data.l[1] == 0)
                     ob_debug_type(OB_DEBUG_APP_BUGS,
                                   "_NET_ACTIVE_WINDOW message for window %s is"
@@ -1334,8 +1331,8 @@ static void event_handle_client(ObClient *client, XEvent *e)
             } else
                 ob_debug_type(OB_DEBUG_APP_BUGS,
                               "_NET_ACTIVE_WINDOW message for window %s is "
-                              "missing source indication\n");
-            client_activate(client, TRUE, TRUE, TRUE,
+                              "missing source indication\n", client->title);
+            client_activate(client, FALSE, FALSE, TRUE, TRUE,
                             (e->xclient.data.l[0] == 0 ||
                              e->xclient.data.l[0] == 2));
         } else if (msgtype == prop_atoms.net_wm_moveresize) {
@@ -1517,6 +1514,13 @@ static void event_handle_client(ObClient *client, XEvent *e)
                reconfigure the window if it needs to. emacs will update its
                normal hints every time it receives a conigurenotify */
             client_reconfigure(client, FALSE);
+        } else if (msgtype == prop_atoms.motif_wm_hints) {
+            client_get_mwm_hints(client);
+            /* This can override some mwm hints */
+            client_get_type_and_transientness(client);
+
+            /* Apply the changes to the window */
+            client_setup_decor_and_functions(client, TRUE);
         } else if (msgtype == XA_WM_HINTS) {
             client_update_wmhints(client);
         } else if (msgtype == XA_WM_TRANSIENT_FOR) {
@@ -1567,9 +1571,22 @@ static void event_handle_client(ObClient *client, XEvent *e)
     default:
         ;
 #ifdef SHAPE
-        if (extensions_shape && e->type == extensions_shape_event_basep) {
-            client->shaped = ((XShapeEvent*)e)->shaped;
-            frame_adjust_shape(client->frame);
+        {
+            int kind;
+            if (extensions_shape && e->type == extensions_shape_event_basep) {
+                switch (((XShapeEvent*)e)->kind) {
+                    case ShapeBounding:
+                    case ShapeClip:
+                        client->shaped = ((XShapeEvent*)e)->shaped;
+                        kind = ShapeBounding;
+                        break;
+                    case ShapeInput:
+                        client->shaped_input = ((XShapeEvent*)e)->shaped;
+                        kind = ShapeInput;
+                        break;
+                }
+                frame_adjust_shape_kind(client->frame, kind);
+            }
         }
 #endif
     }
@@ -1676,30 +1693,30 @@ static gboolean event_handle_menu_keyboard(XEvent *ev)
     else if (ev->type == KeyPress && (state & ~ControlMask) == 0) {
         frame->got_press = TRUE;
 
-        if (keycode == ob_keycode(OB_KEY_ESCAPE)) {
+        if (ob_keycode_match(keycode, OB_KEY_ESCAPE)) {
             menu_frame_hide_all();
             ret = TRUE;
         }
 
-        else if (keycode == ob_keycode(OB_KEY_LEFT)) {
+        else if (ob_keycode_match(keycode, OB_KEY_LEFT)) {
             /* Left goes to the parent menu */
             if (frame->parent)
                 menu_frame_select(frame, NULL, TRUE);
             ret = TRUE;
         }
 
-        else if (keycode == ob_keycode(OB_KEY_RIGHT)) {
+        else if (ob_keycode_match(keycode, OB_KEY_RIGHT)) {
             /* Right goes to the selected submenu */
             if (frame->child) menu_frame_select_next(frame->child);
             ret = TRUE;
         }
 
-        else if (keycode == ob_keycode(OB_KEY_UP)) {
+        else if (ob_keycode_match(keycode, OB_KEY_UP)) {
             menu_frame_select_previous(frame);
             ret = TRUE;
         }
 
-        else if (keycode == ob_keycode(OB_KEY_DOWN)) {
+        else if (ob_keycode_match(keycode, OB_KEY_DOWN)) {
             menu_frame_select_next(frame);
             ret = TRUE;
         }
@@ -1712,7 +1729,7 @@ static gboolean event_handle_menu_keyboard(XEvent *ev)
     else if (ev->type == KeyRelease && (state & ~ControlMask) == 0 &&
              frame->entries && frame->got_press)
     {
-        if (keycode == ob_keycode(OB_KEY_RETURN)) {
+        if (ob_keycode_match(keycode, OB_KEY_RETURN)) {
             /* Enter runs the active item or goes into the submenu.
                Control-Enter runs it without closing the menu. */
             if (frame->child)
@@ -1824,7 +1841,12 @@ static gboolean event_handle_menu(XEvent *ev)
             (f = find_active_menu()) && f->selected == e &&
             e->entry->type != OB_MENU_ENTRY_TYPE_SUBMENU)
         {
-            menu_frame_select(e->frame, NULL, FALSE);
+            ObMenuEntryFrame *u = menu_entry_frame_under(ev->xcrossing.x_root,
+                                                         ev->xcrossing.y_root);
+            /* if we're just going from one entry in the menu to the next,
+               don't unselect stuff first */
+            if (!u || e->frame != u->frame)
+                menu_frame_select(e->frame, NULL, FALSE);
         }
         break;
     case MotionNotify:
@@ -1896,9 +1918,6 @@ static gboolean focus_delay_func(gpointer data)
     ObFocusDelayData *d = data;
     Time old = event_curtime;
 
-    /* don't move focus and kill the menu or the move/resize */
-    if (menu_frame_visible || moveresize_in_progress) return FALSE;
-
     event_curtime = d->time;
     event_curserial = d->serial;
     if (client_focus(d->client) && config_focus_raise)
@@ -1999,7 +2018,7 @@ void event_cancel_all_key_grabs(void)
     XSync(ob_display, FALSE);
 }
 
-gboolean event_time_after(Time t1, Time t2)
+gboolean event_time_after(guint32 t1, guint32 t2)
 {
     g_assert(t1 != CurrentTime);
     g_assert(t2 != CurrentTime);
@@ -2012,8 +2031,10 @@ gboolean event_time_after(Time t1, Time t2)
       - http://tronche.com/gui/x/xlib/input/pointer-grabbing.html
     */
 
-    /* TIME_HALF is half of the number space of a Time type variable */
-#define TIME_HALF (Time)(1 << (sizeof(Time)*8-1))
+    /* TIME_HALF is not half of the number space of a Time type variable.
+     * Rather, it is half the number space of a timestamp value, which is
+     * always 32 bits. */
+#define TIME_HALF (guint32)(1 << 31)
 
     if (t2 >= TIME_HALF)
         /* t2 is in the second half so t1 might wrap around and be smaller than
