@@ -56,11 +56,8 @@ void focus_cycle_stop(ObClient *ifclient)
     /* stop focus cycling if the given client is a valid focus target,
        and so the cycling is being disrupted */
     if (focus_cycle_target && ifclient &&
-        focus_valid_target(ifclient, TRUE,
-                           focus_cycle_iconic_windows,
-                           focus_cycle_all_desktops,
-                           focus_cycle_dock_windows,
-                           focus_cycle_desktop_windows))
+        (ifclient == focus_cycle_target ||
+         focus_cycle_popup_is_showing(ifclient)))
     {
         focus_cycle(TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,TRUE);
         focus_directional_cycle(0, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE);
@@ -122,7 +119,8 @@ ObClient* focus_cycle(gboolean forward, gboolean all_desktops,
                                focus_cycle_iconic_windows,
                                focus_cycle_all_desktops,
                                focus_cycle_dock_windows,
-                               focus_cycle_desktop_windows))
+                               focus_cycle_desktop_windows,
+                               FALSE))
         {
             if (interactive) {
                 if (ft != focus_cycle_target) { /* prevents flicker */
@@ -189,7 +187,7 @@ static ObClient *focus_find_directional(ObClient *c, ObDirection dir,
         if (cur == c)
             continue;
         if (!focus_valid_target(it->data, TRUE, FALSE, FALSE, dock_windows,
-                                desktop_windows))
+                                desktop_windows, FALSE))
             continue;
 
         /* find the centre coords of this window, from the
@@ -297,7 +295,7 @@ ObClient* focus_directional_cycle(ObDirection dir, gboolean dock_windows,
                                    focus_cycle_iconic_windows,
                                    focus_cycle_all_desktops,
                                    focus_cycle_dock_windows,
-                                   focus_cycle_desktop_windows))
+                                   focus_cycle_desktop_windows, FALSE))
                 ft = it->data;
     }
 
