@@ -53,6 +53,9 @@ struct _ObMenuFrame
 
     GList *entries;
     ObMenuEntryFrame *selected;
+    /* if a submenu was selected, then this holds the entry for that submenu
+       until it is closed */
+    ObMenuEntryFrame *open_submenu;
 
     /* show entries from the menu starting at this index */
     guint show_from;
